@@ -8,7 +8,6 @@ use Data::Dumper;
 use File::Basename qw(dirname);
 use Cwd qw(abs_path);
 use File::Spec::Functions qw(catdir catfile);
-use Term::ANSIColor qw(:constants);
 use Moo;
 use Types::Standard qw(Str Int Num Enum ArrayRef HashRef Undef Bool);
 use File::ShareDir::ProjectDistDir qw(dist_dir);
@@ -20,21 +19,15 @@ use Pheno::Ranker::Stats;
 use Exporter 'import';
 our @EXPORT_OK = qw($VERSION write_json);
 
-# Personalize warn and die functions
-$SIG{__WARN__} = sub { warn BOLD YELLOW "Warn: ", @_ };
-$SIG{__DIE__}  = sub { die BOLD RED "Error: ", @_ };
-
 # Global variables:
 $Data::Dumper::Sortkeys = 1;
-our $VERSION   = '0.03_1';
+our $VERSION   = '0.01';
 our $share_dir = dist_dir('Pheno-Ranker');
-
-# Set developoent mode
 use constant DEVEL_MODE => 0;
 
 # Misc variables
 my (
-    $config_sort_by, $config_similarity_metric_cohort, $config_max_out, $config_max_number_var,
+    $config_sort_by, $config_max_out, $config_max_number_var,
     $config_seed,    @config_allowed_terms
 );
 my $default_config_file = catfile( $share_dir, 'conf', 'config.yaml' );
@@ -45,84 +38,70 @@ my $default_config_file = catfile( $share_dir, 'conf', 'config.yaml' );
 
 # Complex defaults here
 has 'config_file' => (
-    is  => 'ro',
-    isa =>
-      sub { die "Config file '$_[0]' is not a valid file" unless -e $_[0] },
     default => $default_config_file,
-    coerce  => sub { $_[0] // $default_config_file },
+    coerce  => sub {
+        $_[0] // $default_config_file;
+    },
+    is      => 'ro',
+    isa     => sub { die "$_[0] is not a valid file" unless -e $_[0] },
     trigger => sub {
         my ( $self, $config_file ) = @_;
         my $config = read_yaml($config_file);
 
-        # Set basic configuration parameters
-        $self->_set_basic_config($config);
+        #####################
+        # Set config params #
+        #####################
 
-        # Validate and set exclusive configuration parameters
-        $self->_validate_and_set_exclusive_config( $config, $config_file );
+        $config_sort_by        = $config->{sort_by}        // 'hamming';
+        $config_max_out        = $config->{max_out}        // 50;
+        $config_max_number_var = $config->{max_number_var} // 10_000;
 
-        # Set additional configuration parameters on $self
-        $self->_set_additional_config( $config, $config_file );
-    }
-);
-
-# Private Method: _set_basic_config
-# Sets basic configuration parameters from the provided config.
-sub _set_basic_config {
-    my ( $self, $config ) = @_;
-    $config_sort_by        = $config->{sort_by}        // 'hamming';
-    $config_similarity_metric_cohort        = $config->{config_similarity_metric_cohort}        // 'hamming';
-    $config_max_out        = $config->{max_out}        // 50;
-    $config_max_number_var = $config->{max_number_var} // 10_000;
-    $config_seed =
-      ( defined $config->{seed} && Int->check( $config->{seed} ) )
-      ? $config->{seed}
-      : 123456789;
-}
-
-# Private Method: _validate_and_set_exclusive_config
-# Validates and sets configuration parameters that are exclusive or conditional.
-sub _validate_and_set_exclusive_config {
-    my ( $self, $config, $config_file ) = @_;
-
-    # Validate $config->{allowed_terms}
-    unless ( exists $config->{allowed_terms}
-        && ArrayRef->check( $config->{allowed_terms} )
-        && @{ $config->{allowed_terms} } )
-    {
-        die "No <allowed terms> provided or not an array ref at $config_file\n";
-    }
-    @config_allowed_terms = @{ $config->{allowed_terms} };
-}
-
-# Private Method: _set_additional_config
-# Sets additional configuration parameters on $self.
-sub _set_additional_config {
-    my ( $self, $config, $config_file ) = @_;
-    $self->{primary_key}              = $config->{primary_key} // 'id';       # setter
-    $self->{exclude_properties_regex} = $config->{exclude_properties_regex}
-      // '';                                                                  # setter
-    $self->{array_terms} = $config->{array_terms} // ['foo'];                 # setter (TBV)
-    $self->{array_regex} = $config->{array_regex} // '^(\w+):(\d+)';          # setter (TBV)
-    $self->{format}      = $config->{format};                                 #setter
-
-    # Validate $config->{id_correspondence} for "real" array_terms
-    if ( $self->{array_terms}[0] ne 'foo' ) {
-        unless ( exists $config->{id_correspondence}
-            && HashRef->check( $config->{id_correspondence} ) )
+        # Validate $config->{allowed_terms}
+        unless ( exists $config->{allowed_terms}
+            && ArrayRef->check( $config->{allowed_terms} )
+            && @{ $config->{allowed_terms} } )
         {
             die
-"No <id_correspondence> provided or not a hash ref at $config_file\n";
+"No <allowed terms> provided or not an array ref at\n$config_file\n";
         }
-        $self->{id_correspondence} = $config->{id_correspondence};
+        @config_allowed_terms = @{ $config->{allowed_terms} };
 
-        # Validate format and check match in config->{id_correspondence}
-        if ( exists $config->{format} && Str->check( $config->{format} ) ) {
-            die
+        ###############################
+        # Set config exclusive params #
+        ###############################
+        $config_seed =
+          ( defined $config->{seed} && Int->check( $config->{seed} ) )
+          ? $config->{seed}
+          : 123456789;
+
+        # Set on $self
+        $self->{primary_key}              = $config->{primary_key} // 'id';    # setter;
+        $self->{exclude_properties_regex} = $config->{exclude_properties_regex}
+          // '';                                                               # setter
+        $self->{array_terms} = $config->{array_terms} // ['foo'];              # setter - To validate
+        $self->{array_regex} = $config->{array_regex} // '^(\w+):(\d+)';       # setter - To validate
+        $self->{format}      = $config->{format};                              # setter
+
+        # Validate $config->{id_correspondence} if we have "real" array_terms
+        if ( $self->{array_terms}[0] ne 'foo' ) {
+            unless ( exists $config->{id_correspondence}
+                && HashRef->check( $config->{id_correspondence} ) )
+            {
+                die
+"No <id_correspondence> provided or not a hash ref at\n$config_file\n";
+            }
+            $self->{id_correspondence} = $config->{id_correspondence};    # setter
+
+            # Validate format if exists and check that has a match in config->{id_correspondence}
+            if ( exists $config->{format} && Str->check( $config->{format} ) ) {
+                die
 "<$config->{format}> does not match any key from <id_correspondence>\n"
-              unless exists $config->{id_correspondence}{ $config->{format} };
+                  unless
+                  exists $config->{id_correspondence}{ $config->{format} };
+            }
         }
     }
-}
+);
 
 has sort_by => (
     default => $config_sort_by,
@@ -132,14 +111,6 @@ has sort_by => (
     isa     => Enum [qw(hamming jaccard)]
 );
 
-has similarity_metric_cohort => (
-    default => $config_similarity_metric_cohort,
-    is      => 'ro',
-    coerce  => sub { $_[0] // $config_similarity_metric_cohort },
-    lazy    => 1,
-    isa     => Enum [qw(hamming jaccard)]
-);
-
 has max_out => (
     default => $config_max_out,                    # Limit to speed up runtime
     is      => 'ro',
@@ -162,7 +133,7 @@ has hpo_file => (
         $_[0] // catfile( $share_dir, 'db', 'hp.json' );
     },
     is  => 'ro',
-    isa => sub { die "Error <$_[0]> is not a valid file" unless -e $_[0] },
+    isa => sub { die "$_[0] is not a valid file" unless -e $_[0] },
 );
 
 has poi_out_dir => (
@@ -171,26 +142,26 @@ has poi_out_dir => (
         $_[0] // catdir('./');
     },
     is  => 'ro',
-    isa => sub { die "<$_[0]> dir does not exist" unless -d $_[0] },
+    isa => sub { die "$_[0] dir does not exist" unless -d $_[0] },
 );
 
 has [qw /include_terms exclude_terms/] => (
     is   => 'ro',
     lazy => 1,
-    isa  => sub {
+
+    #isa     =>  ArrayRef [Enum $config->{allowed_terms}], # It's created at compile time and we don't have $config->{allowed_terms}
+    isa => sub {
         my $value = shift;
-
-        # Ensure the value is an array reference
-        die "<--include_terms> and <--exclude_terms> must be an array ref\n"
+        die "<--include_terms> and <--exclude_terms> must be an array ref"
           unless ref $value eq 'ARRAY';
-
-        # Validate each term against allowed terms
-        foreach my $term (@$value) {
-            die
-"Invalid term '$term' in <--include_terms> or <--exclude_terms>. Allowed values are: "
-              . join( ', ', @config_allowed_terms ) . "\n"
-              unless grep { $_ eq $term } @config_allowed_terms;
+        die
+qq/Invalid term in <--include_terms> or <--exclude_terms>. Allowed values are:\n/,
+          ( join ',', @config_allowed_terms ), "\n"
+          unless all {
+            my $term = $_;
+            grep { $_ eq $term } @config_allowed_terms
         }
+        @$value;
     },
     default => sub { [] },
 );
@@ -224,23 +195,24 @@ sub BUILD {
     # Start Miscellanea checks
     # ************************
 
-    # Check append_prefixes if provided
+    # APPEND_PREFIXES
+    # Check that we have the right numbers of array elements
     if ( @{ $self->{append_prefixes} } ) {
 
-        # Ensure there are more than one reference files
-        die "<--append_prefixes> requires at least 2 cohort files!\n"
+        # die if used without $self->{append_prefixes}
+        die "<--append_prefixes> needs at least 2 cohort files!\n"
           unless @{ $self->{reference_files} } > 1;
 
-        # Ensure numbers of cohorts and append-prefixes match
-        die "The number of items in <--r> and <--append-prefixes> must match!\n"
+        # die if #cohorts and #append-prefixes don't match
+        die "Numbers of items in <--r> and <--append-prefixes> don't match!\n"
           unless @{ $self->{reference_files} } == @{ $self->{append_prefixes} };
     }
 
-    # Check patients_of_interest if provided
+    # PATIENTS-OF-INTEREST
     if ( @{ $self->{patients_of_interest} } ) {
 
-        # Ensure reference files are provided when using patients_of_interest
-        die "<--patients-of-interest> must be used with <--r>\n"
+        # die if used without $self->{append_prefixes}
+        die "<--patients-of-interest> needs to be used with <--r>\n"
           unless @{ $self->{reference_files} };
     }
 
@@ -268,6 +240,8 @@ sub run {
     my $out_file               = $self->{out_file};
     my $cohort_files           = $self->{cohort_files};
     my $append_prefixes        = $self->{append_prefixes};
+    my $max_out                = $self->{max_out};
+    my $sort_by                = $self->{sort_by};
     my $primary_key            = $self->{primary_key};
     my $poi                    = $self->{patients_of_interest};
     my $poi_out_dir            = $self->{poi_out_dir};
@@ -308,7 +282,7 @@ sub run {
     # $ref_data is an array array where each element is the content of the file (e.g, [] or {})
     my $ref_data = [];
     for my $cohort_file ( @{$reference_files} ) {
-        die "<$cohort_file> does not exist\n" unless -f $cohort_file;
+        die "$cohort_file does not exist\n" unless -f $cohort_file;
 
         # Load JSON file as Perl data structure
         my $json_data = io_yaml_or_json(
@@ -321,7 +295,7 @@ sub run {
         # Check for existence of primary_key otherwise die
         my $msg =
 "Sorry, <$cohort_file> does not contain primary_key <$primary_key>. Are you using the right config file?\n";
-        if ( ref $json_data eq ref [] ) {    # array - 1st element only
+        if ( ref $json_data eq ref [] ) {    # array
             die $msg unless exists $json_data->[0]->{$primary_key};
         }
         else {                               # hash
@@ -366,13 +340,6 @@ sub run {
     # We will process $ref_data to get stats on coverage
     my $coverage_stats = coverage_stats($ref_data);
 
-    # Now check existance of include_terms within the data
-    #print Dumper $self->{include_terms} and die;
-    die
-"--include-terms <@{$self->{include_terms}}> does not exist in the cohort(s)\n"
-      unless check_existence_of_include_terms( $coverage_stats,
-        $self->{include_terms} );
-
     # We have to check if we have BFF|PXF or others unless defined at config
     add_attribute( $self, 'format', check_format($ref_data) )
       unless defined $self->{format};    # setter via sub
@@ -481,7 +448,6 @@ sub run {
         }
     }
 
-
     # Dump to JSON if <--export>
     # NB: Must work for -r and -t
     serialize_hashes(
@@ -544,9 +510,9 @@ L<https://github.com/CNAG-Biomedical-Informatics/pheno-ranker#readme>
 
 =head1 CITATION
 
-The author requests that any published work that utilizes C<Pheno-Ranker> includes a cite to the the following reference:
+The author requests that any published work that utilizes C<Convert-Pheno> includes a cite to the the following reference:
 
-Rueda, M. et al. "Advancing Semantic Similarity Analysis of Phenotypic Data Stored in GA4GH Standards and Beyond. (2023) I<Submitted>.
+Rueda, M. et al. "Advancing Semantic Similarity Analysis of Phenotypic Data Stored in GA4GH Standards and Beyond. (2023) I<Manuscript in preparation>.
 
 =head1 AUTHOR
 
@@ -554,11 +520,7 @@ Written by Manuel Rueda, PhD. Info about CNAG can be found at L<https://www.cnag
 
 =head1 METHODS
 
-There is only method named c<run>. See above the syntax.
-
-For more information check the documentation:
-
-L<https://cnag-biomedical-informatics.github.io/pheno-ranker
+See L<https://cnag-biomedical-informatics.github.io/pheno-ranker/use-as-a-module>.
 
 =head1 COPYRIGHT
 
